@@ -8,7 +8,10 @@ module Dalli
     attr_accessor :weight
     attr_accessor :options
     attr_reader :sock
+    attr_reader :socket_type  # possible values: :unix, :tcp
 
+    DEFAULT_PORT = 11211
+    DEFAULT_WEIGHT = 1
     DEFAULTS = {
       # seconds between trying to contact a remote server
       :down_retry_delay => 1,
@@ -32,11 +35,7 @@ module Dalli
     }
 
     def initialize(attribs, options = {})
-      (@hostname, @port, @weight) = attribs.split(':')
-      @port ||= 11211
-      @port = Integer(@port)
-      @weight ||= 1
-      @weight = Integer(@weight)
+      @hostname, @port, @weight, @socket_type = parse_hostname(attribs)
       @fail_count = 0
       @down_at = nil
       @last_down_at = nil
@@ -49,13 +48,17 @@ module Dalli
     end
 
     def name
-      "#{@hostname}:#{@port}"
+      if socket_type == :unix
+        hostname
+      else
+        "#{hostname}:#{port}"
+      end
     end
 
     # Chokepoint method for instrumentation
     def request(op, *args)
       verify_state
-      raise Dalli::NetworkError, "#{hostname}:#{port} is down: #{@error} #{@msg}. If you are sure it is running, ensure memcached version is > 1.4." unless alive?
+      raise Dalli::NetworkError, "#{name} is down: #{@error} #{@msg}. If you are sure it is running, ensure memcached version is > 1.4." unless alive?
       begin
         send(op, *args)
       rescue Dalli::NetworkError
@@ -67,9 +70,10 @@ module Dalli
         false
       rescue Dalli::DalliError
         raise
+      rescue Timeout::Error
+        raise
       rescue => ex
-        Dalli.logger.error "Unexpected exception in Dalli: #{ex.class.name}: #{ex.message}"
-        Dalli.logger.error "This is a bug in Dalli, please enter an issue in Github if it does not already exist."
+        Dalli.logger.error "Unexpected exception during Dalli request: #{ex.class.name}: #{ex.message}"
         Dalli.logger.error ex.backtrace.join("\n\t")
         down!
       end
@@ -80,7 +84,7 @@ module Dalli
 
       if @last_down_at && @last_down_at + options[:down_retry_delay] >= Time.now
         time = @last_down_at + options[:down_retry_delay] - Time.now
-        Dalli.logger.debug { "down_retry_delay not reached for #{hostname}:#{port} (%.3f seconds left)" % time }
+        Dalli.logger.debug { "down_retry_delay not reached for #{name} (%.3f seconds left)" % time }
         return false
       end
 
@@ -203,7 +207,7 @@ module Dalli
     end
 
     def failure!(exception)
-      message = "#{hostname}:#{port} failed (count: #{@fail_count}) #{exception.class}: #{exception.message}"
+      message = "#{name} failed (count: #{@fail_count}) #{exception.class}: #{exception.message}"
       Dalli.logger.info { message }
 
       @fail_count += 1
@@ -223,21 +227,21 @@ module Dalli
 
       if @down_at
         time = Time.now - @down_at
-        Dalli.logger.debug { "#{hostname}:#{port} is still down (for %.3f seconds now)" % time }
+        Dalli.logger.debug { "#{name} is still down (for %.3f seconds now)" % time }
       else
         @down_at = @last_down_at
-        Dalli.logger.warn { "#{hostname}:#{port} is down" }
+        Dalli.logger.warn { "#{name} is down" }
       end
 
       @error = $! && $!.class.name
       @msg = @msg || ($! && $!.message && !$!.message.empty? && $!.message)
-      raise Dalli::NetworkError, "#{hostname}:#{port} is down: #{@error} #{@msg}"
+      raise Dalli::NetworkError, "#{name} is down: #{@error} #{@msg}"
     end
 
     def up!
       if @down_at
         time = Time.now - @down_at
-        Dalli.logger.warn { "#{hostname}:#{port} is back (downtime was %.3f seconds)" % time }
+        Dalli.logger.warn { "#{name} is back (downtime was %.3f seconds)" % time }
       end
 
       @fail_count = 0
@@ -268,6 +272,7 @@ module Dalli
 
     def set(key, value, ttl, cas, options)
       (value, flags) = serialize(key, value, options)
+      ttl = sanitize_ttl(ttl)
 
       guard_max_value(key, value) do
         req = [REQUEST, OPCODES[multi? ? :setq : :set], key.bytesize, 8, 0, 0, value.bytesize + key.bytesize + 8, 0, cas, flags, ttl, key, value].pack(FORMAT[:set])
@@ -278,6 +283,7 @@ module Dalli
 
     def add(key, value, ttl, options)
       (value, flags) = serialize(key, value, options)
+      ttl = sanitize_ttl(ttl)
 
       guard_max_value(key, value) do
         req = [REQUEST, OPCODES[multi? ? :addq : :add], key.bytesize, 8, 0, 0, value.bytesize + key.bytesize + 8, 0, 0, flags, ttl, key, value].pack(FORMAT[:add])
@@ -288,6 +294,7 @@ module Dalli
 
     def replace(key, value, ttl, cas, options)
       (value, flags) = serialize(key, value, options)
+      ttl = sanitize_ttl(ttl)
 
       guard_max_value(key, value) do
         req = [REQUEST, OPCODES[multi? ? :replaceq : :replace], key.bytesize, 8, 0, 0, value.bytesize + key.bytesize + 8, 0, cas, flags, ttl, key, value].pack(FORMAT[:replace])
@@ -308,26 +315,32 @@ module Dalli
       generic_response
     end
 
-    def decr(key, count, ttl, default)
-      expiry = default ? ttl : 0xFFFFFFFF
+    def decr_incr(opcode, key, count, ttl, default)
+      expiry = default ? sanitize_ttl(ttl) : 0xFFFFFFFF
       default ||= 0
       (h, l) = split(count)
       (dh, dl) = split(default)
-      req = [REQUEST, OPCODES[:decr], key.bytesize, 20, 0, 0, key.bytesize + 20, 0, 0, h, l, dh, dl, expiry, key].pack(FORMAT[:decr])
+      req = [REQUEST, OPCODES[opcode], key.bytesize, 20, 0, 0, key.bytesize + 20, 0, 0, h, l, dh, dl, expiry, key].pack(FORMAT[opcode])
       write(req)
       body = generic_response
-      body ? longlong(*body.unpack('NN')) : body
+      body ? body.unpack('Q>').first : body
+    end
+
+    def decr(key, count, ttl, default)
+      decr_incr :decr, key, count, ttl, default
     end
 
     def incr(key, count, ttl, default)
-      expiry = default ? ttl : 0xFFFFFFFF
-      default ||= 0
-      (h, l) = split(count)
-      (dh, dl) = split(default)
-      req = [REQUEST, OPCODES[:incr], key.bytesize, 20, 0, 0, key.bytesize + 20, 0, 0, h, l, dh, dl, expiry, key].pack(FORMAT[:incr])
-      write(req)
-      body = generic_response
-      body ? longlong(*body.unpack('NN')) : body
+      decr_incr :incr, key, count, ttl, default
+    end
+
+    def write_append_prepend(opcode, key, value)
+      write_generic [REQUEST, OPCODES[opcode], key.bytesize, 0, 0, 0, value.bytesize + key.bytesize, 0, 0, key, value].pack(FORMAT[opcode])
+    end
+
+    def write_generic(bytes)
+      write(bytes)
+      generic_response
     end
 
     def write_noop
@@ -343,15 +356,11 @@ module Dalli
     end
 
     def append(key, value)
-      req = [REQUEST, OPCODES[:append], key.bytesize, 0, 0, 0, value.bytesize + key.bytesize, 0, 0, key, value].pack(FORMAT[:append])
-      write(req)
-      generic_response
+      write_append_prepend :append, key, value
     end
 
     def prepend(key, value)
-      req = [REQUEST, OPCODES[:prepend], key.bytesize, 0, 0, 0, value.bytesize + key.bytesize, 0, 0, key, value].pack(FORMAT[:prepend])
-      write(req)
-      generic_response
+      write_append_prepend :prepend, key, value
     end
 
     def stats(info='')
@@ -361,9 +370,7 @@ module Dalli
     end
 
     def reset_stats
-      req = [REQUEST, OPCODES[:stat], 'reset'.bytesize, 0, 0, 0, 'reset'.bytesize, 0, 0, 'reset'].pack(FORMAT[:stat])
-      write(req)
-      generic_response
+      write_generic [REQUEST, OPCODES[:stat], 'reset'.bytesize, 0, 0, 0, 'reset'.bytesize, 0, 0, 'reset'].pack(FORMAT[:stat])
     end
 
     def cas(key)
@@ -373,15 +380,12 @@ module Dalli
     end
 
     def version
-      req = [REQUEST, OPCODES[:version], 0, 0, 0, 0, 0, 0, 0].pack(FORMAT[:noop])
-      write(req)
-      generic_response
+      write_generic [REQUEST, OPCODES[:version], 0, 0, 0, 0, 0, 0, 0].pack(FORMAT[:noop])
     end
 
     def touch(key, ttl)
-      req = [REQUEST, OPCODES[:touch], key.bytesize, 4, 0, 0, key.bytesize + 4, 0, 0, ttl, key].pack(FORMAT[:touch])
-      write(req)
-      generic_response
+      ttl = sanitize_ttl(ttl)
+      write_generic [REQUEST, OPCODES[:touch], key.bytesize, 4, 0, 0, key.bytesize + 4, 0, 0, ttl, key].pack(FORMAT[:touch])
     end
 
     # http://www.hjp.at/zettel/m/memcached_flags.rxml
@@ -396,6 +400,8 @@ module Dalli
         marshalled = true
         begin
           self.serializer.dump(value)
+        rescue TimeoutError => e
+          raise e
         rescue => ex
           # Marshalling can throw several different types of generic Ruby exceptions.
           # Convert to a specific exception so we can special case it higher up the stack.
@@ -434,9 +440,7 @@ module Dalli
     end
 
     def data_cas_response
-      header = read(24)
-      raise Dalli::NetworkError, 'No response' if !header
-      (extras, _, status, count, _, cas) = header.unpack(CAS_HEADER)
+      (extras, _, status, count, _, cas) = read_header.unpack(CAS_HEADER)
       data = read(count) if count > 0
       if status == 1
         nil
@@ -463,10 +467,20 @@ module Dalli
       end
     end
 
+    # https://code.google.com/p/memcached/wiki/NewCommands#Standard_Protocol
+    # > An expiration time, in seconds. Can be up to 30 days. After 30 days, is treated as a unix timestamp of an exact date.
+    MAX_ACCEPTABLE_EXPIRATION_INTERVAL = 30*24*60*60 # 30 days
+    def sanitize_ttl(ttl)
+      if ttl > MAX_ACCEPTABLE_EXPIRATION_INTERVAL
+        Dalli.logger.debug "Expiration interval too long for Memcached, converting to an expiration timestamp"
+        Time.now.to_i + ttl.to_i
+      else
+        ttl.to_i
+      end
+    end
+
     def generic_response(unpack=false)
-      header = read(24)
-      raise Dalli::NetworkError, 'No response' if !header
-      (extras, _, status, count) = header.unpack(NORMAL_HEADER)
+      (extras, _, status, count) = read_header.unpack(NORMAL_HEADER)
       data = read(count) if count > 0
       if status == 1
         nil
@@ -484,9 +498,7 @@ module Dalli
     end
 
     def cas_response
-      header = read(24)
-      raise Dalli::NetworkError, 'No response' if !header
-      (_, _, status, count, _, cas) = header.unpack(CAS_HEADER)
+      (_, _, status, count, _, cas) = read_header.unpack(CAS_HEADER)
       read(count) if count > 0  # this is potential data that we don't care about
       if status == 1
         nil
@@ -502,9 +514,7 @@ module Dalli
     def keyvalue_response
       hash = {}
       loop do
-        header = read(24)
-        raise Dalli::NetworkError, 'No response' if !header
-        (key_length, _, body_length, _) = header.unpack(KV_HEADER)
+        (key_length, _, body_length, _) = read_header.unpack(KV_HEADER)
         return hash if key_length == 0
         key = read(key_length)
         value = read(body_length - key_length) if body_length - key_length > 0
@@ -515,9 +525,7 @@ module Dalli
     def multi_response
       hash = {}
       loop do
-        header = read(24)
-        raise Dalli::NetworkError, 'No response' if !header
-        (key_length, _, body_length, _) = header.unpack(KV_HEADER)
+        (key_length, _, body_length, _) = read_header.unpack(KV_HEADER)
         return hash if key_length == 0
         flags = read(4).unpack('N')[0]
         key = read(key_length)
@@ -548,14 +556,22 @@ module Dalli
       end
     end
 
+    def read_header
+      read(24) || raise(Dalli::NetworkError, 'No response')
+    end
+
     def connect
-      Dalli.logger.debug { "Dalli::Server#connect #{hostname}:#{port}" }
+      Dalli.logger.debug { "Dalli::Server#connect #{name}" }
 
       begin
         @pid = Process.pid
-        @sock = KSocket.open(hostname, port, self, options)
-        @version = version # trigger actual connect
+        if socket_type == :unix
+          @sock = KSocket::UNIX.open(hostname, self, options)
+        else
+          @sock = KSocket::TCP.open(hostname, port, self, options)
+        end
         sasl_authentication if need_auth?
+        @version = version # trigger actual connect
         up!
       rescue Dalli::DalliError # SASL auth failure
         raise
@@ -569,10 +585,6 @@ module Dalli
       [n >> 32, 0xFFFFFFFF & n]
     end
 
-    def longlong(a, b)
-      (a << 32) | b
-    end
-
     REQUEST = 0x80
     RESPONSE = 0x81
 
@@ -661,11 +673,10 @@ module Dalli
       # negotiate
       req = [REQUEST, OPCODES[:auth_negotiation], 0, 0, 0, 0, 0, 0, 0].pack(FORMAT[:noop])
       write(req)
-      header = read(24)
-      raise Dalli::NetworkError, 'No response' if !header
-      (extras, type, status, count) = header.unpack(NORMAL_HEADER)
+
+      (extras, type, status, count) = read_header.unpack(NORMAL_HEADER)
       raise Dalli::NetworkError, "Unexpected message format: #{extras} #{count}" unless extras == 0 && count > 0
-      content = read(count)
+      content = read(count).gsub(/\u0000/, ' ')
       return (Dalli.logger.debug("Authentication not required/supported by server")) if status == 0x81
       mechanisms = content.split(' ')
       raise NotImplementedError, "Dalli only supports the PLAIN authentication mechanism" if !mechanisms.include?('PLAIN')
@@ -676,9 +687,7 @@ module Dalli
       req = [REQUEST, OPCODES[:auth_request], mechanism.bytesize, 0, 0, 0, mechanism.bytesize + msg.bytesize, 0, 0, mechanism, msg].pack(FORMAT[:auth_request])
       write(req)
 
-      header = read(24)
-      raise Dalli::NetworkError, 'No response' if !header
-      (extras, type, status, count) = header.unpack(NORMAL_HEADER)
+      (extras, type, status, count) = read_header.unpack(NORMAL_HEADER)
       raise Dalli::NetworkError, "Unexpected message format: #{extras} #{count}" unless extras == 0 && count > 0
       content = read(count)
       return Dalli.logger.info("Dalli/SASL: #{content}") if status == 0
@@ -688,5 +697,25 @@ module Dalli
       # (step, msg) = sasl.receive('challenge', content)
       # raise Dalli::NetworkError, "Authentication failed" if sasl.failed? || step != 'response'
     end
+
+    def parse_hostname(str)
+      res = str.match(/\A(\[([\h:]+)\]|[^:]+)(?::(\d+))?(?::(\d+))?\z/)
+      raise Dalli::DalliError, "Could not parse hostname #{str}" if res.nil? || res[1] == '[]'
+      hostnam = res[2] || res[1]
+      if hostnam =~ /\A\//
+        socket_type = :unix
+        # in case of unix socket, allow only setting of weight, not port
+        raise Dalli::DalliError, "Could not parse hostname #{str}" if res[4]
+        weigh = res[3]
+      else
+        socket_type = :tcp
+        por = res[3] || DEFAULT_PORT
+        por = Integer(por)
+        weigh = res[4]
+      end
+      weigh ||= DEFAULT_WEIGHT
+      weigh = Integer(weigh)
+      return hostnam, por, weigh, socket_type
+    end
   end
 end
