@@ -23,11 +23,21 @@ describe 'Dalli' do
         end
       end
     end
+
+    it 'return string type for namespace attribute' do
+      dc = Dalli::Client.new('foo', :namespace => :wunderschoen)
+      assert_equal "wunderschoen", dc.send(:namespace)
+      dc.close
+
+      dc = Dalli::Client.new('foo', :namespace => Proc.new{:wunderschoen})
+      assert_equal "wunderschoen", dc.send(:namespace)
+      dc.close
+    end
   end
 
   describe 'key validation' do
     it 'not allow blanks' do
-      memcached do |dc|
+      memcached_persistent do |dc|
         dc.set '   ', 1
         assert_equal 1, dc.get('   ')
         dc.set "\t", 1
@@ -42,6 +52,14 @@ describe 'Dalli' do
         end
       end
     end
+
+    it 'allow namespace to be a symbol' do
+      memcached_persistent do |dc, port|
+        dc = Dalli::Client.new("localhost:#{port}", :namespace => :wunderschoen)
+        dc.set "x" * 251, 1
+        assert 1, dc.get("#{'x' * 200}:md5:#{Digest::MD5.hexdigest('x' * 251)}")
+      end
+    end
   end
 
   it "default to localhost:11211" do
@@ -88,7 +106,7 @@ describe 'Dalli' do
   describe 'using a live server' do
 
     it "support get/set" do
-      memcached do |dc|
+      memcached_persistent do |dc|
         dc.flush
 
         val1 = "1234567890"*105000
@@ -105,7 +123,7 @@ describe 'Dalli' do
     end
 
     it 'supports delete' do
-      memcached do |dc|
+      memcached_persistent do |dc|
         dc.set('some_key', 'some_value')
         assert_equal 'some_value', dc.get('some_key')
 
@@ -115,21 +133,22 @@ describe 'Dalli' do
     end
 
     it 'returns nil for nonexist key' do
-      memcached do |dc|
+      memcached_persistent do |dc|
         assert_equal nil, dc.get('notexist')
       end
     end
 
     it 'allows "Not found" as value' do
-      memcached do |dc|
+      memcached_persistent do |dc|
         dc.set('key1', 'Not found')
         assert_equal 'Not found', dc.get('key1')
       end
     end
 
     it "support stats" do
-      memcached do |dc|
+      memcached_persistent do |dc|
         # make sure that get_hits would not equal 0
+        dc.set(:a, "1234567890"*100000)
         dc.get(:a)
 
         stats = dc.stats
@@ -168,7 +187,7 @@ describe 'Dalli' do
     end
 
     it "support the fetch operation" do
-      memcached do |dc|
+      memcached_persistent do |dc|
         dc.flush
 
         expected = { 'blah' => 'blerg!' }
@@ -191,7 +210,7 @@ describe 'Dalli' do
     end
 
     it "support the fetch operation with falsey values" do
-      memcached do |dc|
+      memcached_persistent do |dc|
         dc.flush
 
         dc.set("fetch_key", false)
@@ -205,7 +224,7 @@ describe 'Dalli' do
     end
 
     it "support the cas operation" do
-      memcached do |dc|
+      memcached_persistent do |dc|
         dc.flush
 
         expected = { 'blah' => 'blerg!' }
@@ -229,7 +248,7 @@ describe 'Dalli' do
     end
 
     it "support multi-get" do
-      memcached do |dc|
+      memcached_persistent do |dc|
         dc.close
         dc.flush
         resp = dc.get_multi(%w(a b c d e f))
@@ -266,7 +285,7 @@ describe 'Dalli' do
     end
 
     it 'support raw incr/decr' do
-      memcached do |client|
+      memcached_persistent do |client|
         client.flush
 
         assert op_addset_succeeds(client.set('fakecounter', 0, 0, :raw => true))
@@ -296,7 +315,7 @@ describe 'Dalli' do
     end
 
     it "support incr/decr operations" do
-      memcached do |dc|
+      memcached_persistent do |dc|
         dc.flush
 
         resp = dc.decr('counter', 100, 5, 0)
@@ -344,7 +363,7 @@ describe 'Dalli' do
     end
 
     it 'support the append and prepend operations' do
-      memcached do |dc|
+      memcached_persistent do |dc|
         dc.flush
         assert op_addset_succeeds(dc.set('456', 'xyz', 0, :raw => true))
         assert_equal true, dc.prepend('456', '0')
@@ -358,7 +377,7 @@ describe 'Dalli' do
     end
 
     it 'supports replace operation' do
-      memcached do |dc|
+      memcached_persistent do |dc|
         dc.flush
         dc.set('key', 'value')
         assert op_replace_succeeds(dc.replace('key', 'value2'))
@@ -368,7 +387,7 @@ describe 'Dalli' do
     end
 
     it 'support touch operation' do
-      memcached do |dc|
+      memcached_persistent do |dc|
         begin
           dc.flush
           dc.set 'key', 'value'
@@ -384,7 +403,7 @@ describe 'Dalli' do
     end
 
     it 'support version operation' do
-      memcached do |dc|
+      memcached_persistent do |dc|
         v = dc.version
         servers = v.keys
         assert(servers.any? do |s|
@@ -394,7 +413,8 @@ describe 'Dalli' do
     end
 
     it 'allow TCP connections to be configured for keepalive' do
-      memcached(19122, '', :keepalive => true) do |dc|
+      memcached_persistent do |dc, port|
+        dc = Dalli::Client.new("localhost:#{port}", :keepalive => true)
         dc.set(:a, 1)
         ring = dc.send(:ring)
         server = ring.servers.first
@@ -408,7 +428,7 @@ describe 'Dalli' do
     end
 
     it "pass a simple smoke test" do
-      memcached do |dc|
+      memcached_persistent do |dc, port|
         resp = dc.flush
         refute_nil resp
         assert_equal [true, true], resp
@@ -436,7 +456,61 @@ describe 'Dalli' do
         dc.close
         dc = nil
 
-        dc = Dalli::Client.new('localhost:19122')
+        dc = Dalli::Client.new("localhost:#{port}")
+
+        assert op_addset_succeeds(dc.set('456', 'xyz', 0, :raw => true))
+
+        resp = dc.prepend '456', '0'
+        assert_equal true, resp
+
+        resp = dc.append '456', '9'
+        assert_equal true, resp
+
+        resp = dc.get('456', :raw => true)
+        assert_equal '0xyz9', resp
+
+        assert op_addset_succeeds(dc.set('456', false))
+
+        resp = dc.get('456')
+        assert_equal false, resp
+
+        resp = dc.stats
+        assert_equal Hash, resp.class
+
+        dc.close
+      end
+    end
+
+    it "pass a simple smoke test on unix socket" do
+      memcached_persistent(MemcachedMock::UNIX_SOCKET_PATH) do |dc, path|
+        resp = dc.flush
+        refute_nil resp
+        assert_equal [true], resp
+
+        assert op_addset_succeeds(dc.set(:foo, 'bar'))
+        assert_equal 'bar', dc.get(:foo)
+
+        resp = dc.get('123')
+        assert_equal nil, resp
+
+        assert op_addset_succeeds(dc.set('123', 'xyz'))
+
+        resp = dc.get('123')
+        assert_equal 'xyz', resp
+
+        assert op_addset_succeeds(dc.set('123', 'abc'))
+
+        dc.prepend('123', '0')
+        dc.append('123', '0')
+
+        assert_raises Dalli::UnmarshalError do
+          resp = dc.get('123')
+        end
+
+        dc.close
+        dc = nil
+
+        dc = Dalli::Client.new(path)
 
         assert op_addset_succeeds(dc.set('456', 'xyz', 0, :raw => true))
 
@@ -462,7 +536,7 @@ describe 'Dalli' do
     end
 
     it "support multithreaded access" do
-      memcached do |cache|
+      memcached_persistent do |cache|
         cache.flush
         workers = []
 
@@ -505,19 +579,26 @@ describe 'Dalli' do
     end
 
     it "handle namespaced keys" do
-      memcached do |dc|
-        dc = Dalli::Client.new('localhost:19122', :namespace => 'a')
+      memcached_persistent do |dc, port|
+        dc = Dalli::Client.new("localhost:#{port}", :namespace => 'a')
         dc.set('namespaced', 1)
-        dc2 = Dalli::Client.new('localhost:19122', :namespace => 'b')
+        dc2 = Dalli::Client.new("localhost:#{port}", :namespace => 'b')
         dc2.set('namespaced', 2)
         assert_equal 1, dc.get('namespaced')
         assert_equal 2, dc2.get('namespaced')
       end
     end
 
+    it "handle nil namespace" do
+      memcached_persistent do |dc, port|
+        dc = Dalli::Client.new("localhost:#{port}", :namespace => nil)
+        assert_equal 'key', dc.send(:validate_key, 'key')
+      end
+    end
+
     it 'truncate cache keys that are too long' do
-      memcached do
-        dc = Dalli::Client.new('localhost:19122', :namespace => 'some:namspace')
+      memcached_persistent do |dc, port|
+        dc = Dalli::Client.new("localhost:#{port}", :namespace => 'some:namspace')
         key = "this cache key is far too long so it must be hashed and truncated and stuff" * 10
         value = "some value"
         assert op_addset_succeeds(dc.set(key, value))
@@ -526,8 +607,8 @@ describe 'Dalli' do
     end
 
     it "handle namespaced keys in multi_get" do
-      memcached do |dc|
-        dc = Dalli::Client.new('localhost:19122', :namespace => 'a')
+      memcached_persistent do |dc, port|
+        dc = Dalli::Client.new("localhost:#{port}", :namespace => 'a')
         dc.set('a', 1)
         dc.set('b', 2)
         assert_equal({'a' => 1, 'b' => 2}, dc.get_multi('a', 'b'))
@@ -535,7 +616,7 @@ describe 'Dalli' do
     end
 
     it "handle application marshalling issues" do
-      memcached do |dc|
+      memcached_persistent do |dc|
         old = Dalli.logger
         Dalli.logger = Logger.new(nil)
         begin
@@ -548,7 +629,7 @@ describe 'Dalli' do
 
     describe 'with compression' do
       it 'allow large values' do
-        memcached do |dc|
+        memcached_persistent do |dc|
           dalli = Dalli::Client.new(dc.instance_variable_get(:@servers), :compress => true)
 
           value = "0"*1024*1024
@@ -561,7 +642,7 @@ describe 'Dalli' do
     describe 'in low memory conditions' do
 
       it 'handle error response correctly' do
-        memcached(19125, '-m 1 -M') do |dc|
+        memcached_low_mem_persistent do |dc|
           failed = false
           value = "1234567890"*100
           1_000.times do |idx|
@@ -578,8 +659,8 @@ describe 'Dalli' do
       end
 
       it 'fit more values with compression' do
-        memcached(19126, '-m 1 -M') do |dc|
-          dalli = Dalli::Client.new('localhost:19126', :compress => true)
+        memcached_low_mem_persistent do |dc, port|
+          dalli = Dalli::Client.new("localhost:#{port}", :compress => true)
           failed = false
           value = "1234567890"*1000
           10_000.times do |idx|
